@@ -20,6 +20,7 @@
  */
 /**
  * SECTION: gesuriclipasset
+ * @title: GESUriClipAsset
  * @short_description: A GESAsset subclass specialized in GESUriClip extraction
  *
  * The #GESUriClipAsset is a special #GESAsset that lets you handle
@@ -37,6 +38,9 @@
 
 static GHashTable *parent_newparent_table = NULL;
 
+static GstDiscoverer *discoverer = NULL;
+static GstDiscoverer *sync_discoverer = NULL;
+
 static void
 initable_iface_init (GInitableIface * initable_iface)
 {
@@ -220,18 +224,37 @@ ges_uri_clip_asset_class_init (GESUriClipAssetClass * klass)
   if (errno)
     timeout = DEFAULT_DISCOVERY_TIMEOUT;
 
-  klass->discoverer = gst_discoverer_new (timeout, &err);
+  if (!discoverer) {
+    discoverer = gst_discoverer_new (timeout, &err);
+    if (!discoverer) {
+      GST_ERROR ("Could not create discoverer: %s", err->message);
+      g_error_free (err);
+      return;
+    }
+  }
+
+  /* The class structure keeps weak pointers on the discoverers so they
+   * can be properly cleaned up in _ges_uri_asset_cleanup(). */
   if (!klass->discoverer) {
-    GST_ERROR ("Could not create discoverer: %s", err->message);
-    g_error_free (err);
-    return;
+    klass->discoverer = discoverer;
+    g_object_add_weak_pointer (G_OBJECT (discoverer),
+        (gpointer *) & klass->discoverer);
+  }
+
+  if (!sync_discoverer) {
+    sync_discoverer = gst_discoverer_new (timeout, &err);
+
+    if (!sync_discoverer) {
+      GST_ERROR ("Could not create discoverer: %s", err->message);
+      g_error_free (err);
+      return;
+    }
   }
 
-  klass->sync_discoverer = gst_discoverer_new (timeout, NULL);
   if (!klass->sync_discoverer) {
-    GST_ERROR ("Could not create discoverer: %s", err->message);
-    g_error_free (err);
-    return;
+    klass->sync_discoverer = sync_discoverer;
+    g_object_add_weak_pointer (G_OBJECT (sync_discoverer),
+        (gpointer *) & klass->sync_discoverer);
   }
 
   g_signal_connect (klass->discoverer, "discovered",
@@ -342,6 +365,32 @@ ges_uri_clip_asset_set_info (GESUriClipAsset * self, GstDiscovererInfo * info)
   priv->info = gst_object_ref (info);
 }
 
+static void
+_set_meta_file_size (const gchar * uri, GESUriClipAsset * asset)
+{
+  GError *error = NULL;
+  GFileInfo *file_info = NULL;
+  guint64 file_size;
+  GFile *gfile = NULL;
+
+  GESMetaContainer *container = GES_META_CONTAINER (asset);
+
+  gfile = g_file_new_for_uri (uri);
+  file_info = g_file_query_info (gfile, "standard::size",
+      G_FILE_QUERY_INFO_NONE, NULL, &error);
+  if (!error) {
+    file_size = g_file_info_get_attribute_uint64 (file_info, "standard::size");
+    ges_meta_container_register_meta_uint64 (container, GES_META_READ_WRITE,
+        "file-size", file_size);
+  } else {
+    g_error_free (error);
+  }
+  if (gfile)
+    g_object_unref (gfile);
+  if (file_info)
+    g_object_unref (file_info);
+}
+
 static void
 _set_meta_foreach (const GstTagList * tags, const gchar * tag,
     GESMetaContainer * container)
@@ -371,6 +420,8 @@ discoverer_discovered_cb (GstDiscoverer * discoverer,
   if (tags)
     gst_tag_list_foreach (tags, (GstTagForeachFunc) _set_meta_foreach, mfs);
 
+  _set_meta_file_size (uri, mfs);
+
   if (gst_discoverer_info_get_result (info) == GST_DISCOVERER_OK) {
     ges_uri_clip_asset_set_info (mfs, info);
   } else {
@@ -546,6 +597,8 @@ ges_uri_clip_asset_request_sync (const gchar * uri, GError ** error)
         gst_discoverer_info_get_result (info));
   }
 
+  _set_meta_file_size (uri, asset);
+
   ges_asset_cache_put (gst_object_ref (asset), NULL);
   ges_uri_clip_asset_set_info (asset, info);
   ges_asset_cache_set_loaded (GES_TYPE_URI_CLIP, uri, lerror);
@@ -602,6 +655,7 @@ ges_uri_clip_asset_get_stream_assets (GESUriClipAsset * self)
  *****************************************************************/
 /**
  * SECTION: gesurisourceasset
+ * @title: GESUriClipAsset
  * @short_description: A GESAsset subclass specialized in GESUriSource extraction
  *
  * NOTE: You should never request such a #GESAsset as they will be created automatically
@@ -711,3 +765,10 @@ ges_uri_source_asset_get_filesource_asset (GESUriSourceAsset * asset)
 
   return asset->priv->parent_asset;
 }
+
+void
+_ges_uri_asset_cleanup (void)
+{
+  g_clear_object (&discoverer);
+  g_clear_object (&sync_discoverer);
+}
